﻿using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Lib3D;

namespace Extrude
{
    public partial class Form1 : Form
    {
        private Scene2D _Scene2D;
        private Scene3D _Scene3D;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            _Scene2D = new Scene2D(_View2D);
            _Scene3D = new Scene3D(_View3D);

            // just to test cloning
            _Scene2D = _Scene2D.Clone() as Scene2D;
            _Scene3D = _Scene3D.Clone() as Scene3D;

            _Timer.Start();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            _Scene2D.Dispose();
            _Scene3D.Dispose();
        }

        private void _SwapWButton_Click(object sender, EventArgs e)
        {
            _Scene2D.SwapWinding();
        }

        private void _ClearButton_Click(object sender, EventArgs e)
        {
            _Scene3D.Clear();
            _Scene2D.Clear();
        }

        private void _LoadBgButton_Click(object sender, EventArgs e)
        {
            if (_OpenPolyDialog.ShowDialog() == DialogResult.OK)
                _Scene2D.Load(_OpenPolyDialog.FileName, true);
        }

        private void _LoadPolyButton_Click(object sender, EventArgs e)
        {
            if (_OpenPolyDialog.ShowDialog() == DialogResult.OK)
            {
                _Scene3D.Clear();
                _Scene2D.Load(_OpenPolyDialog.FileName, false);
            }
        }


        private void _SavePolyButton_Click(object sender, EventArgs e)
        {
            if (_SavePolyDialog.ShowDialog() == DialogResult.OK)
                _Scene2D.Save(_SavePolyDialog.FileName);
        }

        private void _ExtrudeButton_Click(object sender, EventArgs e)
        {
            double height = double.Parse(_ExtHeightText.Text);
            double s = double.Parse(_TexScaleText.Text);

            _Scene2D.Extrude(_Scene3D, height, s);
        }

        private void _SaveOBJButton_Click(object sender, EventArgs e)
        {
            if (_SaveOBJDialog.ShowDialog() == DialogResult.OK)
                _Scene3D.Save(_SaveOBJDialog.FileName);
        }

        private void _LoadTexButton_Click(object sender, EventArgs e)
        {
            if (_LoadTexDialog.ShowDialog() == DialogResult.OK)
                _Scene3D.SetTexture(_LoadTexDialog.FileName);
        }

        private void _Timer_Tick(object sender, EventArgs e)
        {
            _Scene2D.Draw();
            _Scene3D.Draw();
        }

        private void _View2D_MouseDown(object sender, MouseEventArgs e)
        {
            _Scene2D.MouseDown(e, _SnapCheck.Checked, _AddRadio.Checked, _DeleteRadio.Checked);
        }

        private void _View2D_MouseUp(object sender, MouseEventArgs e)
        {
            _Scene2D.MouseUp(e);
        }

        private void _View2D_MouseMove(object sender, MouseEventArgs e)
        {
            _Scene2D.MouseMove(e, _SnapCheck.Checked, _CameraRadio.Checked, _MoveRadio.Checked);
        }

        private void _View3D_MouseDown(object sender, MouseEventArgs e)
        {
            _Scene3D.MouseDown(e);
        }

        private void _View3D_MouseUp(object sender, MouseEventArgs e)
        {
            _Scene3D.MouseUp(e);
        }

        private void _View3D_MouseMove(object sender, MouseEventArgs e)
        {
            _Scene3D.MouseMove(e);
        }
    }
}
