﻿using System;
using System.Collections.Generic;
using Lib3D;

namespace Extrude
{
    public class Polygon : SceneNode
    {
        public Polygon(GraphicsSystem graphicsSystem)
            : base(graphicsSystem)
        {
            Sel = -1;
            Points = new List<Vector2>();
            Materials[0].LightingEnabled = false;
            Materials[0].BlendEnabled = false;
            Materials[0].DiffuseColor = new Vector4(0, 0, 1, 1);
            Materials[0].LineWidth = 1;
            Materials[0].PointSize = 4;
            Materials[0].Pass = RenderPass.Immediate;
            Materials.Add(new Material(GraphicsSystem));
            Materials[1].LightingEnabled = false;
            Materials[1].BlendEnabled = false;
            Materials[1].DiffuseColor = new Vector4(0, 0, 1, 1);
            Materials[1].LineWidth = 1;
            Materials[1].PointSize = 4;
            Materials[1].Pass = RenderPass.Immediate;
        }

        public Polygon(Polygon polygon)
            : base(polygon)
        {
            Sel = polygon.Sel;
            Points = new List<Vector2>(polygon.Points);
        }

        public int Sel
        {
            get;
            set;
        }

        public List<Vector2> Points
        {
            get;
            set;
        }

        public void SwapWinding()
        {
            SwapWinding(Points);
            CreateGeometryBuffers(true);
        }

        public void SwapWinding(List<Vector2> points)
        {
            int i = 0;
            int j = points.Count - 1;

            while (i < j)
            {
                Vector2 temp = points[i];

                points[i] = points[j];
                points[j] = temp;
                i++;
                j--;
            }
        }

        public double GetArea()
        {
            double area = 0;

            for (int i = 0; i != Points.Count; i++)
            {
                Vector2 a = Points[i];
                Vector2 b = Points[(i + 1) % Points.Count];

                area += a.X * b.Y;
                area -= a.Y * b.X;
            }
            return area / 2;
        }

        public void CreateGeometryBuffers(bool setWindingColor)
        {
            Vector4 color = new Vector4(0.4, 0.4, 0.4, 1);

            Materials[0].GeometryBuffer = CreateLineBuffer();
            Materials[1].GeometryBuffer = CreatePointBuffer();

            if (setWindingColor)
                color = (GetArea() < 0) ? new Vector4(0, 0, 1, 1) : new Vector4(1, 0, 0, 1);

            foreach (Material material in Materials)
                material.DiffuseColor = color;
        }

        private GeometryBuffer CreateLineBuffer()
        {
            GeometryBuffer geometryBuffer = CreateGeometryBuffer();


            geometryBuffer.PrimitiveType = PrimitiveType.Lines;
            for (int i = 0; i != geometryBuffer.Vertices.Count; i++)
            {
                geometryBuffer.Indices.Add(i);
                geometryBuffer.Indices.Add((i + 1) % geometryBuffer.Vertices.Count);
            }
            return geometryBuffer;
        }

        private GeometryBuffer CreatePointBuffer()
        {
            GeometryBuffer geometryBuffer = CreateGeometryBuffer();

            geometryBuffer.PrimitiveType = PrimitiveType.Points;
            for (int i = 0; i != geometryBuffer.Vertices.Count; i++)
                geometryBuffer.Indices.Add(i);
            return geometryBuffer;
        }

        private GeometryBuffer CreateGeometryBuffer()
        {
            GeometryBuffer geometryBuffer = new GeometryBuffer(GraphicsSystem);

            for (int i = 0; i != Points.Count; i++)
            {
                Vector2 p = Points[i];

                geometryBuffer.Vertices.Add(new Vertex(new Vector3(p.X, 1, p.Y), new Vector3(), new Vector2()));
            }
            return geometryBuffer;
        }
    }
}
