﻿using System;
using System.Windows.Forms;
using Lib3D;

namespace Extrude
{
    public abstract class Scene : SceneNode, IDisposable
    {
        public Scene(Scene scene)
            : base(scene)
        {
            GraphicsSystem.RootNode = this;

            InitializeCamera();
            Button = MouseButtons.None;
            MouseLast = new Vector2(0, 0);
        }

        public Scene(Control viewControl)
            : base(new GraphicsSystem(viewControl))
        {
            GraphicsSystem.RootNode = this;
            GraphicsSystem.ClearColor = new Vector4(0.6, 0.6, 0.5, 1);

            InitializeCamera();

            Button = MouseButtons.None;
            MouseLast = new Vector2(0, 0);
        }

        protected Camera Camera
        {
            get { return GraphicsSystem.Camera; }
        }

        protected Control ViewControl
        {
            get { return GraphicsSystem.ViewControl; }
        }

        protected MouseButtons Button
        {
            get;
            set;
        }

        protected Vector2 MouseLast
        {
            get;
            set;
        }

        public abstract void Clear();

        protected abstract void InitializeCamera();

        public abstract void Draw();

        public virtual void Dispose()
        {
            GraphicsSystem.Dispose();
        }
    }
}
