﻿using System;
using System.Collections.Generic;

namespace Graph
{
    public class Evaluator
    {
        public List<Tree> Equations
        {
            get;
            set;
        }

        public double U
        {
            get;
            set;
        }

        public double V
        {
            get;
            set;
        }

        public double X
        {
            get;
            set;
        }

        public double Y
        {
            get;
            set;
        }

        public double Z
        {
            get;
            set;
        }

        public void Eval()
        {
            X = 0;
            Y = 0;
            Z = 0;
            foreach (Tree equation in Equations)
                EvalEquation(equation);
        }

        private void EvalEquation(Tree equation)
        {
            TokenType var = equation.Token.Type;
            Tree expr = equation.Children[0];

            if (var == TokenType.U)
                U = Eval(expr);
            else if (var == TokenType.V)
                V = Eval(expr);
            else if (var == TokenType.X)
                X = Eval(expr);
            else if (var == TokenType.Y)
                Y = Eval(expr);
            else if (var == TokenType.Z)
                Z = Eval(expr);
            else
                throw new TokenException("Bug in code : should have been variable!", equation.Token);
        }

        private double Eval(Tree tree)
        {
            Token token = tree.Token;

            if (token.Type == TokenType.Plus)
                return Eval(tree.Children[0]) + Eval(tree.Children[1]);
            else if (token.Type == TokenType.Minus)
            {
                if (tree.Children.Count == 1)
                    return -Eval(tree.Children[0]);
                else
                    return Eval(tree.Children[0]) - Eval(tree.Children[1]);
            }
            else if (token.Type == TokenType.Times)
                return Eval(tree.Children[0]) * Eval(tree.Children[1]);
            else if (token.Type == TokenType.Divide)
                return Eval(tree.Children[0]) / Eval(tree.Children[1]);
            else if (token.Type == TokenType.Pow)
                return Math.Pow(Eval(tree.Children[0]), Eval(tree.Children[1]));
            else if (token.Type == TokenType.Sin)
                return Math.Sin(Eval(tree.Children[0]));
            else if (token.Type == TokenType.Cos)
                return Math.Cos(Eval(tree.Children[0]));
            else if (token.Type == TokenType.Tan)
                return Math.Tan(Eval(tree.Children[0]));
            else if (token.Type == TokenType.ASin)
                return Math.Asin(Eval(tree.Children[0]));
            else if (token.Type == TokenType.ACos)
                return Math.Acos(Eval(tree.Children[0]));
            else if (token.Type == TokenType.ATan)
                return Math.Atan(Eval(tree.Children[0]));
            else if (token.Type == TokenType.Sqrt)
                return Math.Sqrt(Eval(tree.Children[0]));
            else if (token.Type == TokenType.Abs)
                return Math.Abs(Eval(tree.Children[0]));
            else if (token.Type == TokenType.Log)
                return Math.Log(Eval(tree.Children[0]));
            else if (token.Type == TokenType.Ceiling)
                return Math.Ceiling(Eval(tree.Children[0]));
            else if (token.Type == TokenType.Floor)
                return Math.Floor(Eval(tree.Children[0]));
            else if (token.Type == TokenType.Min)
                return Math.Min(Eval(tree.Children[0]), Eval(tree.Children[1]));
            else if (token.Type == TokenType.Max)
                return Math.Max(Eval(tree.Children[0]), Eval(tree.Children[1]));
            else if (token.Type == TokenType.U)
                return U;
            else if (token.Type == TokenType.V)
                return V;
            else if (token.Type == TokenType.X)
                return X;
            else if (token.Type == TokenType.Y)
                return Y;
            else if (token.Type == TokenType.Z)
                return Z;
            else if (token.Type == TokenType.E)
                return Math.E;
            else if (token.Type == TokenType.PI)
                return Math.PI;
            else if (token.Type == TokenType.Number)
                return token.Number;
            else
                throw new TokenException("Bug in code : should have been evaluated!", token);
        }
    }
}
