﻿using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Lib3D;

namespace Graph
{
    public partial class Form1 : Form
    {
        GraphicsSystem _GraphicsSystem;
        TextureLoader _TexLoader;
        Evaluator _Evaluator;
        Parser _Parser;
        MouseButtons _Buttons;
        Vector2 _Last;
        int _UDivs;
        int _VDivs;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            _GraphicsSystem = new GraphicsSystem(_View);
            _GraphicsSystem.RootNode = new SceneNode(_GraphicsSystem);
            _GraphicsSystem.RootNode.AddChild(new TransformSceneNode(_GraphicsSystem));
            _GraphicsSystem.RootNode.First.Materials[0].Name = "mat1";
            _GraphicsSystem.RootNode.First.Materials[0].SpecularColor = new Vector4(1, 1, 1, 1);
            _GraphicsSystem.RootNode.First.Materials[0].Power = 100;
            _GraphicsSystem.RootNode.AddChild(new LightSceneNode(_GraphicsSystem, new Vector3(100, 100, 100), new Vector4(1, 1, 1, 1), 0.001));
            _GraphicsSystem.RootNode.AddChild(new LightSceneNode(_GraphicsSystem, new Vector3(-100, -100, -100), new Vector4(1, 1, 1, 1), 0.001));
            _GraphicsSystem.ClearColor = new Vector4(1, 1, 1, 1);
            _TexLoader = new TextureLoader(_GraphicsSystem);
            _Evaluator = new Evaluator();
            _Parser = new Parser();
            _Buttons = MouseButtons.None;
            _Last = new Vector2(0, 0);
            _UDivsCombo.SelectedIndex = _UDivsCombo.Items.IndexOf("128");
            _VDivsCombo.SelectedIndex = _VDivsCombo.Items.IndexOf("128");
            _UDivs = 0;
            _VDivs = 0;

            InitCamera();
            SetColor();

            _Timer.Start();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            _TexLoader.Dispose();
            _GraphicsSystem.RootNode.First.Materials[0].GeometryBuffer.Dispose();
            _GraphicsSystem.Dispose();
        }

        private void _ColorView_Click(object sender, EventArgs e)
        {
            if (_ColorPicker.ShowDialog() == DialogResult.OK)
            {
                _ColorView.BackColor = _ColorPicker.Color;
                SetColor();
            }
        }

        private void _LoadTexButton_Click(object sender, EventArgs e)
        {
            if (_OpenTexDialog.ShowDialog() == DialogResult.OK)
            {
                _TexLoader.Clear();
                _GraphicsSystem.RootNode.First.Materials[0].Texture = _TexLoader.Get(_OpenTexDialog.FileName);
            }
        }

        private void _ClearTexButton_Click(object sender, EventArgs e)
        {
            _TexLoader.Clear();
            _GraphicsSystem.RootNode.First.Materials[0].Texture = null;
        }

        private void _SaveOBJButton_Click(object sender, EventArgs e)
        {
            if (_SaveOBJDialog.ShowDialog() == DialogResult.OK)
                Save(_SaveOBJDialog.FileName);
        }

        private void _SaveButton_Click(object sender, EventArgs e)
        {
            if (_SaveGraphDialog.ShowDialog() == DialogResult.OK)
                File.WriteAllText(_SaveGraphDialog.FileName, _GraphText.Text);
        }

        private void _LoadButton_Click(object sender, EventArgs e)
        {
            if (_OpenGraphDialog.ShowDialog() == DialogResult.OK)
                _GraphText.Text = File.ReadAllText(_OpenGraphDialog.FileName);
        }

        private void _PlotButton_Click(object sender, EventArgs e)
        {
            GeometryBuffer graph = _GraphicsSystem.RootNode.First.Materials[0].GeometryBuffer;
            int uDivs = int.Parse(_UDivsCombo.Items[_UDivsCombo.SelectedIndex].ToString());
            int vDivs = int.Parse(_VDivsCombo.Items[_VDivsCombo.SelectedIndex].ToString());
            List<Vertex> vertices = new List<Vertex>(uDivs * vDivs);
            List<int> indices = new List<int>((uDivs - 1) * (vDivs - 1) * 6);

            _ErrorLabel.Text = ":";
            try
            {
                _Evaluator.Equations = _Parser.Parse(new Tokenizer(_GraphText.Text));
            }
            catch (TokenException ex)
            {
                _ErrorLabel.Text = ex.Message;
                MessageBox.Show("Error : " + ex.Message);

                return;
            }
            graph.Dispose();

            for(int i = 0; i != uDivs; i++)
                for (int j = 0; j != vDivs; j++)
                {
                    double u = i / (uDivs - 1.0f);
                    double v = j / (vDivs - 1.0f);

                    _Evaluator.U = u;
                    _Evaluator.V = v;
                    _Evaluator.Eval();

                    double x = _Evaluator.X;
                    double y = _Evaluator.Y;
                    double z = _Evaluator.Z;

                    vertices.Add(new Vertex(new Vector3(x, y, z), new Vector3(), new Vector2(u, v)));
                }
            for(int i = 0; i != uDivs - 1; i++)
                for (int j = 0; j != vDivs - 1; j++)
                {
                    indices.Add(i * vDivs + j);
                    indices.Add(i * vDivs + j + 1);
                    indices.Add((i + 1) * vDivs + j + 1);
                    indices.Add((i + 1) * vDivs + j + 1);
                    indices.Add((i + 1) * vDivs + j);
                    indices.Add(i * vDivs + j);
                }
            graph.Vertices = vertices;
            graph.Indices = indices;
            graph.CalcNormals();
            graph.FlipNormals();
            graph.Compile();
            graph.CalcBounds();

            TransformSceneNode graphNode = _GraphicsSystem.RootNode.First as TransformSceneNode;
            AABBox bounds = graph.Bounds;
            Vector3 dim = bounds.Dimension;
            double max = Math.Max(Math.Max(dim.X, dim.Y), dim.Z);
            Vector3 scale = new Vector3(100, 100, 100) / max;
            Matrix scaleMatrix = Matrix.CreateScaling(scale);

            bounds = scaleMatrix * bounds;
            graphNode.Position = -bounds.Center;
            graphNode.Scale = scale;

            InitCamera();

            _UDivs = uDivs;
            _VDivs = vDivs;
        }

        private void _Timer_Tick(object sender, EventArgs e)
        {
            _GraphicsSystem.Draw();
        }

        private void _View_MouseDown(object sender, MouseEventArgs e)
        {
            _Last = new Vector2(e.X, e.Y);
            _Buttons = e.Button;
        }

        private void _View_MouseUp(object sender, MouseEventArgs e)
        {
            _Buttons = MouseButtons.None;
        }

        private void _View_MouseMove(object sender, MouseEventArgs e)
        {
            if (_Buttons == MouseButtons.None)
                return;

            Vector2 mouse = new Vector2(e.X, e.Y);
            Vector2 d = mouse - _Last;

            _GraphicsSystem.Camera.RotateAroundTarget(d);
            _Last = mouse;
        }

        private void InitCamera()
        {
            _GraphicsSystem.Camera.Position = new Vector3(70, 70, 70);
            _GraphicsSystem.Camera.Target = new Vector3(0, 0, 0);
            _GraphicsSystem.Camera.Up = new Vector3(-1, 1, -1);
        }

        private void SetColor()
        {
            Color c = _ColorView.BackColor;
            Vector4 ambient = new Vector4(0.2 * c.R / 255.0, 0.2 * c.G / 255.0, 0.2 * c.B / 255.0, 1);
            Vector4 diffuse = new Vector4(c.R / 255.0, c.G / 255.0, c.B / 255.0, 1);

            _GraphicsSystem.RootNode.First.Materials[0].AmbientColor = ambient;
            _GraphicsSystem.RootNode.First.Materials[0].DiffuseColor = diffuse;
        }

        private void _SwapWButton_Click(object sender, EventArgs e)
        {
            GeometryBuffer graph = _GraphicsSystem.RootNode.First.Materials[0].GeometryBuffer;

            graph.SwapWinding();
            graph.Compile();
        }


        private void _FlipNButton_Click(object sender, EventArgs e)
        {
            GeometryBuffer graph = _GraphicsSystem.RootNode.First.Materials[0].GeometryBuffer;

            graph.FlipNormals();
            graph.Compile();
        }

        private void _TexScaleButton_Click(object sender, EventArgs e)
        {
            GeometryBuffer graph = _GraphicsSystem.RootNode.First.Materials[0].GeometryBuffer;
            Vector2 scale = new Vector2(_TexScale.Text);
            int uDivs = _UDivs;
            int vDivs = _VDivs;

            for (int i = 0, k = 0; i != uDivs; i++)
                for (int j = 0; j != vDivs; j++, k++)
                {
                    double u = i / (uDivs - 1.0f);
                    double v = j / (vDivs - 1.0f);
                    Vertex vertex = graph.Vertices[k];

                    vertex.TextureCoordinate = new Vector2(u, v) * scale;
                    graph.Vertices[k] = vertex;
                }
            graph.Compile();
        }

        private void Save(string file)
        {
            OBJFile.Write(file, _GraphicsSystem.RootNode.First.Materials);
        }

        private void _CullCheck_CheckedChanged(object sender, EventArgs e)
        {
            Material material = _GraphicsSystem.RootNode.First.Materials[0];

            if (_CullCheck.Checked)
                material.CullMode = CullMode.Clockwise;
            else
                material.CullMode = CullMode.None;
        }
    }
}
