﻿using System;
using System.Collections.Generic;

namespace Graph
{
    public class Parser
    {
        private Tokenizer _Tokenizer;

        public Parser()
        {
            _Tokenizer = null;
        }

        public List<Tree> Parse(Tokenizer tokenizer)
        {
            List<Tree> equations = new List<Tree>();

            _Tokenizer = tokenizer;
            while (_Tokenizer.HasNext())
                equations.Add(ParseEquation());

            return equations;
        }

        private Tree ParseEquation()
        {
            Token token = _Tokenizer.Next();
            Tree equation = new Tree(token);

            if (token.Type != TokenType.U &&
                token.Type != TokenType.V &&
                token.Type != TokenType.X &&
                token.Type != TokenType.Y &&
                token.Type != TokenType.Z
                )
                throw new TokenException("Expected variable", token);
            token = _Tokenizer.Next();
            if (token.Type != TokenType.Eq)
                throw new TokenException("Expected '='", token);
            equation.Children.Add(ParseExpr1());

            return equation;
        }

        private Tree ParseExpr1()
        {
            Tree a = ParseExpr2();

            while (_Tokenizer.HasNext())
            {
                Token op = _Tokenizer.Peek();

                if (op.Type == TokenType.Plus || op.Type == TokenType.Minus)
                {
                    Tree b = new Tree(_Tokenizer.Next());

                    b.Children.Add(a);
                    b.Children.Add(ParseExpr2());
                    a = b;
                }
                else
                    break;
            }
            return a;
        }

        private Tree ParseExpr2()
        {
            Tree a = ParseExpr3();

            while (_Tokenizer.HasNext())
            {
                Token op = _Tokenizer.Peek();

                if (op.Type == TokenType.Times || op.Type == TokenType.Divide)
                {
                    Tree b = new Tree(_Tokenizer.Next());

                    b.Children.Add(a);
                    b.Children.Add(ParseExpr3());
                    a = b;
                }
                else
                    break;
            }
            return a;
        }

        private Tree ParseExpr3()
        {
            Tree a = ParseExpr4();

            while (_Tokenizer.HasNext())
            {
                Token op = _Tokenizer.Peek();

                if (op.Type == TokenType.Pow)
                {
                    Tree b = new Tree(_Tokenizer.Next());

                    b.Children.Add(a);
                    b.Children.Add(ParseExpr4());
                    a = b;
                }
                else
                    break;
            }
            return a;
        }

        private Tree ParseExpr4()
        {
            Token token = _Tokenizer.Peek();
            Tree tree;

            if (token.Type == TokenType.Minus ||
                token.Type == TokenType.Sin ||
                token.Type == TokenType.Cos ||
                token.Type == TokenType.Tan ||
                token.Type == TokenType.ASin ||
                token.Type == TokenType.ACos ||
                token.Type == TokenType.ATan ||
                token.Type == TokenType.Sqrt ||
                token.Type == TokenType.Abs ||
                token.Type == TokenType.Log ||
                token.Type == TokenType.Ceiling ||
                token.Type == TokenType.Floor
                )
            {
                tree = new Tree(_Tokenizer.Next());
                tree.Children.Add(ParseExpr5());
            }
            else if (token.Type == TokenType.Min || token.Type == TokenType.Max)
            {
                tree = new Tree(_Tokenizer.Next());
                tree.Children.Add(ParseExpr5());
                token = _Tokenizer.Next();
                if (token.Type != TokenType.Comma)
                    throw new TokenException("Expected ','", token);
                tree.Children.Add(ParseExpr5());
            }
            else
                tree = ParseExpr5();
            return tree;
        }

        private Tree ParseExpr5()
        {
            Token token = _Tokenizer.Next();
            Tree tree;

            if (token.Type == TokenType.LParen)
            {
                tree = ParseExpr1();
                token = _Tokenizer.Next();
                if (token.Type != TokenType.RParen)
                    throw new TokenException("Expected ','", token);
            }
            else if (token.Type == TokenType.U ||
                token.Type == TokenType.V ||
                token.Type == TokenType.X ||
                token.Type == TokenType.Y ||
                token.Type == TokenType.Z ||
                token.Type == TokenType.E ||
                token.Type == TokenType.PI ||
                token.Type == TokenType.Number)
                tree = new Tree(token);
            else
                throw new TokenException("Expected '(', variable, constant or number", token);
            return tree;
        }
    }
}
