﻿using System;

namespace Lib3D
{
    public class Camera
    {
        public Camera()
        {
            OrthoProjection = false;
            Position = new Vector3(0, 200, 200);
            Target = new Vector3(0, 0, 0);
            Up = new Vector3(0, 1, -1);
            FOV = 60;
            Aspect = 800.0f / 600.0f;
            NearPlane = 1;
            FarPlane = 5000;
            Left = -400;
            Right = 400;
            Bottom = -400;
            Top = 400;
        }

        public bool OrthoProjection
        {
            get;
            set;
        }

        public Vector3 Position
        {
            get;
            set;
        }

        public Vector3 Target
        {
            get;
            set;
        }

        public Vector3 Up
        {
            get;
            set;
        }

        public double FOV
        {
            get;
            set;
        }

        public double Aspect
        {
            get;
            set;
        }

        public double NearPlane
        {
            get;
            set;
        }

        public double FarPlane
        {
            get;
            set;
        }

        public double Left
        {
            get;
            set;
        }

        public double Right
        {
            get;
            set;
        }

        public double Bottom
        {
            get;
            set;
        }

        public double Top
        {
            get;
            set;
        }

        public Matrix ViewMatrix
        {
            get { return Matrix.CreateLookAt(Position, Target, Vector3.Normalize(Up)); }
        }

        public Matrix ProjectionMatrix
        {
            get
            {
                if (OrthoProjection)
                    return Matrix.CreateOrtho(Left, Right, Bottom, Top, NearPlane, FarPlane);
                else
                    return Matrix.CreatePerspective(FOV, Aspect, NearPlane, FarPlane); 
            }
        }

        public void RotateAroundTarget(Vector2 d)
        {
            Vector3 f = Position - Target;
            Vector3 t = f;
            Vector3 r;
            Vector3 u = Up;
            Matrix rt = Matrix.CreateFromAxisAngle(new Vector3(0, 1, 0), -d.X);

            f.Normalize();
            u.Normalize();
            r = Vector3.Cross(f, u);
            f = rt * f;
            r = rt * r;
            r.Normalize();
            u = Vector3.Cross(r, f);
            u.Normalize();
            rt = Matrix.CreateFromAxisAngle(r, d.Y);
            f = rt * f;
            u = rt * u;
            f.Normalize();
            u.Normalize();
            Up = u;
            Position = Target + f * t.Length();
        }

        public void RotateAroundPosition(Vector2 d)
        {
            Vector3 f = Target - Position;
            Vector3 t = f;
            Vector3 r;
            Vector3 u = Up;
            Matrix rt = Matrix.CreateFromAxisAngle(new Vector3(0, 1, 0), -d.X);

            f.Normalize();
            u.Normalize();
            r = Vector3.Cross(f, u);
            f = rt * f;
            r = rt * r;
            r.Normalize();
            u = Vector3.Cross(r, f);
            u.Normalize();
            rt = Matrix.CreateFromAxisAngle(r, -d.Y);
            f = rt * f;
            u = rt * u;
            f.Normalize();
            u.Normalize();
            Up = u;
            Target = Position + f * t.Length();
        }
    }
}
