﻿using System;

namespace Lib3D
{
    public struct Frustum
    {
        public Plane Left;
        public Plane Right;
        public Plane Bottom;
        public Plane Top;
        public Plane Near;
        public Plane Far;

        public Frustum(Matrix matrix)
        {
            Matrix m = matrix;

            Left = new Plane(new Vector3(m.M30 + m.M00, m.M31 + m.M01, m.M32 + m.M02), m.M33 + m.M03);
            Left.Normalize();

            Right = new Plane(new Vector3(m.M30 - m.M00, m.M31 - m.M01, m.M32 - m.M02), m.M33 - m.M03);
            Right.Normalize();

            Bottom = new Plane(new Vector3(m.M30 + m.M10, m.M31 + m.M11, m.M32 + m.M12), m.M33 + m.M13);
            Bottom.Normalize();

            Top = new Plane(new Vector3(m.M30 - m.M10, m.M31 - m.M11, m.M32 - m.M12), m.M33 - m.M13);
            Top.Normalize();

            Near = new Plane(new Vector3(m.M30 + m.M20, m.M31 + m.M21, m.M32 + m.M22), m.M33 + m.M23);
            Near.Normalize();

            Far = new Plane(new Vector3(m.M30 - m.M20, m.M31 - m.M21, m.M32 - m.M22), m.M33 - m.M23);
            Far.Normalize();
        }

        public bool Contains(AABBox box)
        {
            if (box.IsEmpty)
                return true;
            else if (Left.Classify(box) < 0)
                return false;
            else if (Right.Classify(box) < 0)
                return false;
            else if (Bottom.Classify(box) < 0)
                return false;
            else if (Top.Classify(box) < 0)
                return false;
            else if (Near.Classify(box) < 0)
                return false;
            else if (Far.Classify(box) < 0)
                return false;
            else
                return true;
        }
    }
}
