﻿using System;
using System.Collections.Generic;

namespace Lib3D
{
    public enum RenderPass
    {
        Background,
        Solid,
        Transparent,
        Immediate
    }

    public enum BlendFactor
    {
        Zero = 0,
        One = 1,
        SourceAlpha = 0x0302,
        InverseSourceAlpha = 0x0303,
        SourceColor = 0x0300,
        InverseSourceColor = 0x0301,
        DestinationAlpha = 0x0304,
        InverseDestinationAlpha = 0x0305,
        DestinationColor = 0x0306,
        InverseDestinationeColor = 0x0307,
    }

    public enum CullMode
    {
        None,
        Clockwise,
        CounterClockwise
    }

    public class Material
    {
        public Material(GraphicsSystem graphicsSystem)
        {
            Name = "";
            Enabled = true;
            LightingEnabled = true;
            BlendEnabled = false;
            ColorMaterialEnabled = false;
            Pass = RenderPass.Solid;
            SourceBlend = BlendFactor.SourceAlpha;
            DestBlend = BlendFactor.InverseSourceAlpha;
            CullMode = CullMode.Clockwise;
            PointSize = 1;
            LineWidth = 1;
            AmbientColor = new Vector4(0.2, 0.2, 0.2, 1);
            DiffuseColor = new Vector4(0.6, 0.6, 0.6, 1);
            SpecularColor = new Vector4(0, 0, 0, 1);
            EmissiveColor = new Vector4(0, 0, 0, 1);
            Power = 25;
            GeometryBuffer = new GeometryBuffer(graphicsSystem);
            TextureEnabled = true;
            Texture = null;
            TextureMatrix = Matrix.CreateIdentity();
        }

        public Material(Material material)
        {
            Name = material.Name;
            Enabled = material.Enabled;
            LightingEnabled = material.LightingEnabled;
            BlendEnabled = material.BlendEnabled;
            ColorMaterialEnabled = material.ColorMaterialEnabled;
            Pass = material.Pass;
            SourceBlend = material.SourceBlend;
            DestBlend = material.DestBlend;
            CullMode = material.CullMode;
            PointSize = material.PointSize;
            LineWidth = material.LineWidth;
            AmbientColor = material.AmbientColor;
            DiffuseColor = material.DiffuseColor;
            SpecularColor = material.SpecularColor;
            EmissiveColor = material.EmissiveColor;
            Power = material.Power;
            GeometryBuffer = new GeometryBuffer(material.GeometryBuffer);
            TextureEnabled = material.TextureEnabled;
            Texture = material.Texture;
            TextureMatrix = material.TextureMatrix;
        }

        public GraphicsSystem GraphicsSystem
        {
            get { return GeometryBuffer.GraphicsSystem; } 
        }

        public string Name
        {
            get;
            set;
        }

        public bool Enabled
        {
            get;
            set;
        }

        public bool LightingEnabled
        {
            get;
            set;
        }

        public bool BlendEnabled
        {
            get;
            set;
        }

        public bool ColorMaterialEnabled
        {
            get;
            set;
        }

        public RenderPass Pass
        {
            get;
            set;
        }

        public BlendFactor SourceBlend
        {
            get;
            set;
        }

        public BlendFactor DestBlend
        {
            get;
            set;
        }

        public CullMode CullMode
        {
            get;
            set;
        }

        public double LineWidth
        {
            get;
            set;
        }

        public double PointSize
        {
            get;
            set;
        }

        public Vector4 AmbientColor
        {
            get;
            set;
        }

        public Vector4 DiffuseColor
        {
            get;
            set;
        }

        public Vector4 SpecularColor
        {
            get;
            set;
        }

        public Vector4 EmissiveColor
        {
            get;
            set;
        }

        public double Power
        {
            get;
            set;
        }

        public GeometryBuffer GeometryBuffer
        {
            get;
            set;
        }

        public bool TextureEnabled
        {
            get;
            set;
        }

        public Texture Texture
        {
            get;
            set;
        }

        public Matrix TextureMatrix
        {
            get;
            set;
        }
    }
}
