﻿using System;

namespace Lib3D
{
    public struct Matrix
    {
        public double M00;
        public double M01;
        public double M02;
        public double M03;
        public double M10;
        public double M11;
        public double M12;
        public double M13;
        public double M20;
        public double M21;
        public double M22;
        public double M23;
        public double M30;
        public double M31;
        public double M32;
        public double M33;

        public Matrix(
            double m00, double m01, double m02, double m03,
            double m10, double m11, double m12, double m13,
            double m20, double m21, double m22, double m23,
            double m30, double m31, double m32, double m33
            )
        {
            M00 = m00;
            M01 = m01;
            M02 = m02;
            M03 = m03;
            M10 = m10;
            M11 = m11;
            M12 = m12;
            M13 = m13;
            M20 = m20;
            M21 = m21;
            M22 = m22;
            M23 = m23;
            M30 = m30;
            M31 = m31;
            M32 = m32;
            M33 = m33;
        }

        public Matrix(string matrix)
        {
            string[] tokens = matrix.Split(new char[] { ' ', '\t', '\r', '\n', ',' }, StringSplitOptions.RemoveEmptyEntries);
            int i = 0;

            M00 = double.Parse(tokens[i++]);
            M01 = double.Parse(tokens[i++]);
            M02 = double.Parse(tokens[i++]);
            M03 = double.Parse(tokens[i++]);
            M10 = double.Parse(tokens[i++]);
            M11 = double.Parse(tokens[i++]);
            M12 = double.Parse(tokens[i++]);
            M13 = double.Parse(tokens[i++]);
            M20 = double.Parse(tokens[i++]);
            M21 = double.Parse(tokens[i++]);
            M22 = double.Parse(tokens[i++]);
            M23 = double.Parse(tokens[i++]);
            M30 = double.Parse(tokens[i++]);
            M31 = double.Parse(tokens[i++]);
            M32 = double.Parse(tokens[i++]);
            M33 = double.Parse(tokens[i++]);
        }

        public Vector3 Translation
        {
            get { return new Vector3(M03, M13, M23); }
        }

        public void CopyTo(double[] array)
        {
            array[0] = M00;
            array[4] = M01;
            array[8] = M02;
            array[12] = M03;
            array[1] = M10;
            array[5] = M11;
            array[9] = M12;
            array[13] = M13;
            array[2] = M20;
            array[6] = M21;
            array[10] = M22;
            array[14] = M23;
            array[3] = M30;
            array[7] = M31;
            array[11] = M32;
            array[15] = M33;
        }

        public double Determinant()
        {
            return
                M03 * M12 * M21 * M30 - M02 * M13 * M21 * M30 -
                M03 * M11 * M22 * M30 + M01 * M13 * M22 * M30 +
                M02 * M11 * M23 * M30 - M01 * M12 * M23 * M30 -
                M03 * M12 * M20 * M31 + M02 * M13 * M20 * M31 +
                M03 * M10 * M22 * M31 - M00 * M13 * M22 * M31 -
                M02 * M10 * M23 * M31 + M00 * M12 * M23 * M31 +
                M03 * M11 * M20 * M32 - M01 * M13 * M20 * M32 -
                M03 * M10 * M21 * M32 + M00 * M13 * M21 * M32 +
                M01 * M10 * M23 * M32 - M00 * M11 * M23 * M32 -
                M02 * M11 * M20 * M33 + M01 * M12 * M20 * M33 +
                M02 * M10 * M21 * M33 - M00 * M12 * M21 * M33 -
                M01 * M10 * M22 * M33 + M00 * M11 * M22 * M33;
        }

        public Vector3 Unproject(double winX, double winY, double winZ, double viewWidth, double viewHeight)
        {
            return Invert(this) * new Vector3(2 * winX / viewWidth - 1, 2 * winY / viewHeight - 1, 2 * winZ - 1);
        }

        public Vector4 ToQuaternion()
        {
            Vector4 q;
            double w4;

            q.W = Math.Sqrt(1.0 + M00 + M11 + M22) / 2.0;
            w4 = (4.0 * q.W);
            q.X = (M21 - M12) / w4;
            q.Y = (M02 - M20) / w4;
            q.Z = (M10 - M01) / w4;

            return q;
        }

        public static Matrix operator *(Matrix a, Matrix b)
        {
            return new Matrix(
                a.M00 * b.M00 + a.M01 * b.M10 + a.M02 * b.M20 + a.M03 * b.M30,
                a.M00 * b.M01 + a.M01 * b.M11 + a.M02 * b.M21 + a.M03 * b.M31,
                a.M00 * b.M02 + a.M01 * b.M12 + a.M02 * b.M22 + a.M03 * b.M32,
                a.M00 * b.M03 + a.M01 * b.M13 + a.M02 * b.M23 + a.M03 * b.M33,
                a.M10 * b.M00 + a.M11 * b.M10 + a.M12 * b.M20 + a.M13 * b.M30,
                a.M10 * b.M01 + a.M11 * b.M11 + a.M12 * b.M21 + a.M13 * b.M31,
                a.M10 * b.M02 + a.M11 * b.M12 + a.M12 * b.M22 + a.M13 * b.M32,
                a.M10 * b.M03 + a.M11 * b.M13 + a.M12 * b.M23 + a.M13 * b.M33,
                a.M20 * b.M00 + a.M21 * b.M10 + a.M22 * b.M20 + a.M23 * b.M30,
                a.M20 * b.M01 + a.M21 * b.M11 + a.M22 * b.M21 + a.M23 * b.M31,
                a.M20 * b.M02 + a.M21 * b.M12 + a.M22 * b.M22 + a.M23 * b.M32,
                a.M20 * b.M03 + a.M21 * b.M13 + a.M22 * b.M23 + a.M23 * b.M33,
                a.M30 * b.M00 + a.M31 * b.M10 + a.M32 * b.M20 + a.M33 * b.M30,
                a.M30 * b.M01 + a.M31 * b.M11 + a.M32 * b.M21 + a.M33 * b.M31,
                a.M30 * b.M02 + a.M31 * b.M12 + a.M32 * b.M22 + a.M33 * b.M32,
                a.M30 * b.M03 + a.M31 * b.M13 + a.M32 * b.M23 + a.M33 * b.M33
                );
        }

        public static Vector2 operator *(Matrix a, Vector2 b)
        {
            return new Vector2(
                a.M00 * b.X + a.M01 * b.Y + a.M03,
                a.M10 * b.X + a.M11 * b.Y + a.M13
                );
        }

        public static Vector3 operator *(Matrix a, Vector3 b)
        {
            return new Vector3(
                a.M00 * b.X + a.M01 * b.Y + a.M02 * b.Z + a.M03,
                a.M10 * b.X + a.M11 * b.Y + a.M12 * b.Z + a.M13,
                a.M20 * b.X + a.M21 * b.Y + a.M22 * b.Z + a.M23
                );
        }

        public static Vector4 operator *(Matrix a, Vector4 b)
        {
            return new Vector4(
                a.M00 * b.X + a.M01 * b.Y + a.M02 * b.Z + a.M03 * b.W,
                a.M10 * b.X + a.M11 * b.Y + a.M12 * b.Z + a.M13 * b.W,
                a.M20 * b.X + a.M21 * b.Y + a.M22 * b.Z + a.M23 * b.W,
                a.M30 * b.X + a.M31 * b.Y + a.M32 * b.Z + a.M33 * b.W
                );
        }

        public static Matrix CreateIdentity()
        {
            return new Matrix(
                1, 0, 0, 0,
                0, 1, 0, 0,
                0, 0, 1, 0,
                0, 0, 0, 1
                );
        }

        public static Matrix CreateTranslation(Vector3 translation)
        {
            return new Matrix(
                1, 0, 0, translation.X,
                0, 1, 0, translation.Y,
                0, 0, 1, translation.Z,
                0, 0, 0, 1
                );
        }

        public static Matrix CreateFromAxisAngle(Vector3 axis, double degrees)
        {
            double radians = degrees * (double)Math.PI / 180;
            double c = Math.Cos(radians);
            double s = Math.Sin(radians);
            double x, y, z;

            axis.Normalize();
            x = axis.X;
            y = axis.Y;
            z = axis.Z;

            return new Matrix(
                x * x * (1 - c) + c, x * y * (1 - c) - z * s, x * z * (1 - c) + y * s, 0,
                y * x * (1 - c) + z * s, y * y * (1 - c) + c, y * z * (1 - c) - x * s, 0,
                x * z * (1 - c) - y * s, y * z * (1 - c) + x * s, z * z * (1 - c) + c, 0,
                0, 0, 0, 1
                );
        }

        public static Matrix CreateScaling(Vector3 scale)
        {
            return new Matrix(
                scale.X, 0, 0, 0,
                0, scale.Y, 0, 0,
                0, 0, scale.Z, 0,
                0, 0, 0, 1
                );
        }

        public static Matrix CreatePerspective(double fovDegrees, double aspect, double near, double far)
        {
            double radians = fovDegrees * (double)Math.PI / 180;
            double f = 1 / Math.Tan(radians / 2);
            return new Matrix(
                f / aspect,     0,      0,                              0,
                0,              f,      0,                              0,
                0,              0,      (far + near) / (near - far),    2 * far * near / (near - far),
                0,              0,      -1,                             0
                );
        }

        public static Matrix CreateOrtho(double left, double right, double bottom, double top, double near, double far)
        {
            double x = -(right + left) / (right - left);
            double y = -(top + bottom) / (top - bottom);
            double z = -(far + near) / (far - near);

            return new Matrix(
                2 / (right - left), 0,                      0,                      x,
                0,                  2 / (top - bottom),     0,                      y,
                0,                  0,                      -2 / (far - near),      z,
                0,                  0,                      0,                      1
                );
        }

        public static Matrix CreateLookAt(Vector3 position, Vector3 target, Vector3 up)
        {
            Vector3 f = Vector3.Normalize(target - position);
            Vector3 u = Vector3.Normalize(up);
            Vector3 r = Vector3.Normalize(Vector3.Cross(f, u));

            u = Vector3.Normalize(Vector3.Cross(r, f));
            f = -f;

            return new Matrix(
                r.X, r.Y, r.Z, -Vector3.Dot(r, position),
                u.X, u.Y, u.Z, -Vector3.Dot(u, position),
                f.X, f.Y, f.Z, -Vector3.Dot(f, position),
                0, 0, 0, 1
                );
        }

        public static Matrix CreateFromQuaternion(Vector4 quaternion)
        {
            Vector4 q = Vector4.Normalize(quaternion);

            return new Matrix(
                1 - 2 * q.Y * q.Y - 2 * q.Z * q.Z, 2 * q.X * q.Y - 2 * q.Z * q.W, 2 * q.X * q.Z + 2 * q.Y * q.W, 0,
                2 * q.X * q.Y + 2 * q.Z * q.W, 1 - 2 * q.X * q.X - 2 * q.Z * q.Z, 2 * q.Y * q.Z - 2 * q.X * q.W, 0,
                2 * q.X * q.Z - 2 * q.Y * q.W, 2 * q.Y * q.Z + 2 * q.X * q.W, 1 - 2 * q.X * q.X - 2 * q.Y * q.Y, 0,
                0, 0, 0, 1
            );
        }

        public static Matrix Transpose(Matrix m)
        {
            return new Matrix(
                m.M00, m.M10, m.M20, m.M30,
                m.M01, m.M11, m.M21, m.M31,
                m.M02, m.M12, m.M22, m.M32,
                m.M03, m.M13, m.M23, m.M33
                );
        }

        public static Matrix Invert(Matrix m)
        {
            double d = m.Determinant();

            if (d == 0)
                d = 1;
            else
                d = 1 / d;

            return new Matrix(
            d * (m.M12 * m.M23 * m.M31 - m.M13 * m.M22 * m.M31 + m.M13 * m.M21 * m.M32 - m.M11 * m.M23 * m.M32 - m.M12 * m.M21 * m.M33 + m.M11 * m.M22 * m.M33),
            d * (m.M03 * m.M22 * m.M31 - m.M02 * m.M23 * m.M31 - m.M03 * m.M21 * m.M32 + m.M01 * m.M23 * m.M32 + m.M02 * m.M21 * m.M33 - m.M01 * m.M22 * m.M33),
            d * (m.M02 * m.M13 * m.M31 - m.M03 * m.M12 * m.M31 + m.M03 * m.M11 * m.M32 - m.M01 * m.M13 * m.M32 - m.M02 * m.M11 * m.M33 + m.M01 * m.M12 * m.M33),
            d * (m.M03 * m.M12 * m.M21 - m.M02 * m.M13 * m.M21 - m.M03 * m.M11 * m.M22 + m.M01 * m.M13 * m.M22 + m.M02 * m.M11 * m.M23 - m.M01 * m.M12 * m.M23),
            d * (m.M13 * m.M22 * m.M30 - m.M12 * m.M23 * m.M30 - m.M13 * m.M20 * m.M32 + m.M10 * m.M23 * m.M32 + m.M12 * m.M20 * m.M33 - m.M10 * m.M22 * m.M33),
            d * (m.M02 * m.M23 * m.M30 - m.M03 * m.M22 * m.M30 + m.M03 * m.M20 * m.M32 - m.M00 * m.M23 * m.M32 - m.M02 * m.M20 * m.M33 + m.M00 * m.M22 * m.M33),
            d * (m.M03 * m.M12 * m.M30 - m.M02 * m.M13 * m.M30 - m.M03 * m.M10 * m.M32 + m.M00 * m.M13 * m.M32 + m.M02 * m.M10 * m.M33 - m.M00 * m.M12 * m.M33),
            d * (m.M02 * m.M13 * m.M20 - m.M03 * m.M12 * m.M20 + m.M03 * m.M10 * m.M22 - m.M00 * m.M13 * m.M22 - m.M02 * m.M10 * m.M23 + m.M00 * m.M12 * m.M23),
            d * (m.M11 * m.M23 * m.M30 - m.M13 * m.M21 * m.M30 + m.M13 * m.M20 * m.M31 - m.M10 * m.M23 * m.M31 - m.M11 * m.M20 * m.M33 + m.M10 * m.M21 * m.M33),
            d * (m.M03 * m.M21 * m.M30 - m.M01 * m.M23 * m.M30 - m.M03 * m.M20 * m.M31 + m.M00 * m.M23 * m.M31 + m.M01 * m.M20 * m.M33 - m.M00 * m.M21 * m.M33),
            d * (m.M01 * m.M13 * m.M30 - m.M03 * m.M11 * m.M30 + m.M03 * m.M10 * m.M31 - m.M00 * m.M13 * m.M31 - m.M01 * m.M10 * m.M33 + m.M00 * m.M11 * m.M33),
            d * (m.M03 * m.M11 * m.M20 - m.M01 * m.M13 * m.M20 - m.M03 * m.M10 * m.M21 + m.M00 * m.M13 * m.M21 + m.M01 * m.M10 * m.M23 - m.M00 * m.M11 * m.M23),
            d * (m.M12 * m.M21 * m.M30 - m.M11 * m.M22 * m.M30 - m.M12 * m.M20 * m.M31 + m.M10 * m.M22 * m.M31 + m.M11 * m.M20 * m.M32 - m.M10 * m.M21 * m.M32),
            d * (m.M01 * m.M22 * m.M30 - m.M02 * m.M21 * m.M30 + m.M02 * m.M20 * m.M31 - m.M00 * m.M22 * m.M31 - m.M01 * m.M20 * m.M32 + m.M00 * m.M21 * m.M32),
            d * (m.M02 * m.M11 * m.M30 - m.M01 * m.M12 * m.M30 - m.M02 * m.M10 * m.M31 + m.M00 * m.M12 * m.M31 + m.M01 * m.M10 * m.M32 - m.M00 * m.M11 * m.M32),
            d * (m.M01 * m.M12 * m.M20 - m.M02 * m.M11 * m.M20 + m.M02 * m.M10 * m.M21 - m.M00 * m.M12 * m.M21 - m.M01 * m.M10 * m.M22 + m.M00 * m.M11 * m.M22)
            );
        }
    }
}
