﻿using System;
using System.IO;
using System.Collections.Generic;

namespace Lib3D
{
    public class OBJFile
    {
        public static void Write(string file, List<Material> materials)
        {
            List<string> lines = new List<string>(materials.Count * 6 + 1);
            string directory = Path.GetDirectoryName(file);
            string mtlFile = Path.GetFileName(file).Replace(Path.GetExtension(file), ".mtl");

            lines.Add("# Material for - " + file);
            foreach (Material material in materials)
            {
                lines.Add("newmtl " + material.Name);
                lines.Add(string.Format("Ka {0} {1} {2}", material.AmbientColor.X, material.AmbientColor.Y, material.AmbientColor.Z));
                lines.Add(string.Format("Kd {0} {1} {2}", material.DiffuseColor.X, material.DiffuseColor.Y, material.DiffuseColor.Z));
                lines.Add(string.Format("Ks {0} {1} {2}", material.SpecularColor.X, material.SpecularColor.Y, material.SpecularColor.Z));
                lines.Add("Ns " + material.Power);
                if (material.Texture != null)
                {
                    string texFile = Path.GetFileName(material.Texture.File);
                    string srcFile = material.Texture.File;
                    string dstFile = Path.Combine(directory, texFile);

                    srcFile = srcFile.ToLower();
                    dstFile = dstFile.ToLower();
                    if (srcFile != dstFile)
                        File.Copy(srcFile, dstFile);
                    lines.Add("map_Kd " + texFile);
                }
                else
                    lines.Add("");
            }
            File.WriteAllLines(Path.Combine(directory, mtlFile), lines.ToArray());

            int count = 2;
            int x = 1;

            foreach (Material material in materials)
                count += material.GeometryBuffer.Vertices.Count * 3 + material.GeometryBuffer.Indices.Count / 3 + 1;
            lines = new List<string>(count);
            lines.Add("# Created by Lib3D Dot Net Library - McNamara Consulting, Inc. @2010");
            lines.Add("mtllib " + mtlFile);
            foreach (Material material in materials)
            {
                GeometryBuffer geometry = material.GeometryBuffer;

                foreach (Vertex vertex in geometry.Vertices)
                    lines.Add(string.Format("v {0} {1} {2}", vertex.Position.X, vertex.Position.Y, vertex.Position.Z));
                foreach (Vertex vertex in geometry.Vertices)
                    lines.Add(string.Format("vn {0} {1} {2}", vertex.Normal.X, vertex.Normal.Y, vertex.Normal.Z));
                foreach (Vertex vertex in geometry.Vertices)
                    lines.Add(string.Format("vt {0} {1}", vertex.TextureCoordinate.X, vertex.TextureCoordinate.Y));
                lines.Add("usemtl " + material.Name);
                for (int i = 0; i != geometry.Indices.Count; )
                {
                    int a = geometry.Indices[i++] + x;
                    int b = geometry.Indices[i++] + x;
                    int c = geometry.Indices[i++] + x;

                    lines.Add(string.Format("f {0}/{0}/{0} {1}/{1}/{1} {2}/{2}/{2}", a, b, c));
                }
                x += geometry.Vertices.Count;
            }
            File.WriteAllLines(file, lines.ToArray());
        }
    }
}
