﻿using System;
using System.IO;
using System.Collections.Generic;

namespace Lib3D
{
    public class OBJSceneNode : TransformSceneNode
    {
        public OBJSceneNode(string file, TextureLoader loader)
            : base(loader.GraphicsSystem)
        {
            string[] lines = File.ReadAllLines(file);
            string materialName = "";
            List<Vector3> v = new List<Vector3>();
            List<Vector3> n = new List<Vector3>();
            List<Vector2> t = new List<Vector2>();
            Material defMaterial = new Material(loader.GraphicsSystem);
            Dictionary<string, Material> materials = new Dictionary<string, Material>();
            string directory = Path.GetDirectoryName(file);

            Name = Path.GetFileName(file);
            for (int i = 0; i != lines.Length; i++)
            {
                string line = lines[i].Trim();

                if (line.StartsWith("v "))
                    v.Add(new Vector3(line.Substring(1)));
                else if (line.StartsWith("vn "))
                    n.Add(new Vector3(line.Substring(2)));
                else if (line.StartsWith("vt "))
                    t.Add(new Vector2(line.Substring(2)));
                else if (line.StartsWith("mtllib "))
                    AddMaterials(line.Substring(6).Trim(), materials, directory, loader);
                else if (line.StartsWith("usemtl "))
                    materialName = line.Substring(6).Trim();
                else if (line.StartsWith("f "))
                    AddFace(materials, v, n, t, materialName, defMaterial, line.Substring(1).Trim());
            }
            Materials.Add(defMaterial);
            foreach (Material material in materials.Values)
                Materials.Add(material);
        }

        public OBJSceneNode(OBJSceneNode sceneNode)
            : base(sceneNode)
        {
        }

        private void AddMaterials(string file, Dictionary<string, Material> materials, string directory, TextureLoader loader)
        {
            Material material;

            file = Path.Combine(directory, file);

            if (!File.Exists(file))
                return;

            string[] lines = File.ReadAllLines(file);
            string materialName = "";

            for (int i = 0; i != lines.Length; i++)
            {
                string line = lines[i].Trim();

                if (line.StartsWith("newmtl "))
                {
                    materialName = line.Substring(6).Trim();
                    if (!materials.ContainsKey(materialName))
                    {
                        materials.Add(materialName, material = new Material(loader.GraphicsSystem));
                        material.Name = materialName;
                    }
                }
                else if (line.StartsWith("Ka "))
                    materials[materialName].AmbientColor = new Vector4(line.Substring(2));
                else if (line.StartsWith("Kd "))
                    materials[materialName].DiffuseColor = new Vector4(line.Substring(2));
                else if (line.StartsWith("Ks "))
                    materials[materialName].SpecularColor = new Vector4(line.Substring(2));
                else if (line.StartsWith("Ns "))
                    materials[materialName].Power = double.Parse(line.Substring(2));
                else if (line.StartsWith("map_Kd "))
                    materials[materialName].Texture = loader.Get(Path.Combine(directory, line.Substring(6).Trim()));
            }
        }

        private void AddFace(
            Dictionary<string, Material> materials, 
            List<Vector3> v, 
            List<Vector3> n, 
            List<Vector2> t, 
            string materialName,
            Material defMaterial,
            string line)
        {
            string[] tokens = line.Split(new char[] { ' ', '\t' }, StringSplitOptions.RemoveEmptyEntries);
            Material material = defMaterial;
            int i = 0;

            if (materials.ContainsKey(materialName))
                material = materials[materialName];

            int x = material.GeometryBuffer.Vertices.Count;
            int count = 0;

            for (i = 0; i != tokens.Length; i++)
            {
                string[] indices = tokens[i].Split(new char[] { '/' }, StringSplitOptions.None);
                Vertex vertex;

                if (indices.Length == 3)
                {
                    if (indices[1] == "")
                        vertex = new Vertex(v[int.Parse(indices[0]) - 1], n[int.Parse(indices[2]) - 1], new Vector2());
                    else
                        vertex = new Vertex(v[int.Parse(indices[0]) - 1], n[int.Parse(indices[2]) - 1], t[int.Parse(indices[1]) - 1]);
                }
                else
                    vertex = new Vertex(v[int.Parse(indices[0]) - 1], new Vector3(), t[int.Parse(indices[1]) - 1]);
                material.GeometryBuffer.Vertices.Add(vertex);
                count++;
            }

            int tris = count - 2;

            i = 0;
            while (i != tris)
            {
                material.GeometryBuffer.Indices.Add(x);
                material.GeometryBuffer.Indices.Add(x + i + 1);
                material.GeometryBuffer.Indices.Add(x + i + 2);
                i++;
            }
        }
    }
}
