﻿using System;

namespace Lib3D
{
    public struct Plane
    {
        public Vector3 Normal;
        public double D;

        public Plane(double a, double b, double c, double d)
        {
            Normal = new Vector3(a, b, c);
            D = d;
        }

        public Plane(Vector3 normal, double d)
        {
            Normal = normal;
            D = d;
        }

        public Plane(Vector3 a, Vector3 b, Vector3 c)
        {
            Normal = Vector3.Normalize(Vector3.Cross(c - b, b - a));
            D = Vector3.Dot(a, Normal);
        }

        public void Normalize()
        {
            double length = Normal.Length();

            Normal /= length;
            D /= length;
        }

        public int Classify(AABBox box)
        {
            double minD = D;
            double maxD = D;

            minD += (Normal.X < 0) ? Normal.X * box.Max.X : Normal.X * box.Min.X;
            maxD += (Normal.X > 0) ? Normal.X * box.Max.X : Normal.X * box.Min.X;

            minD += (Normal.Y < 0) ? Normal.Y * box.Max.Y : Normal.Y * box.Min.Y;
            maxD += (Normal.Y > 0) ? Normal.Y * box.Max.Y : Normal.Y * box.Min.Y;

            minD += (Normal.Z < 0) ? Normal.Z * box.Max.Z : Normal.Z * box.Min.Z;
            maxD += (Normal.Z > 0) ? Normal.Z * box.Max.Z : Normal.Z * box.Min.Z;

            if (minD < 0 && maxD < 0)
                return -1;
            else if (minD > 0 && maxD > 0)
                return 1;
            else
                return 0;
        }

        public Nullable<double> ISect(Ray ray)
        {
            double t = Vector3.Dot(ray.Direction, Normal);

            if (t != 0)
                return (-D - Vector3.Dot(ray.Origin, Normal)) / t;
            return null;
        }
    }
}
