﻿using System;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Drawing.Imaging;

namespace Lib3D
{
    public class Texture : IDisposable
    {
        private GraphicsSystem _GraphicsSystem;
        private int _Width;
        private int _Height;
        private uint _ID;
        private string _File;

        public Texture(GraphicsSystem graphicsSystem, string file)
        {
            Bitmap bitmap = null;

            _File = file;

            _GraphicsSystem = graphicsSystem;
            try
            {
                CreateTexture(bitmap = Bitmap.FromFile(file) as Bitmap);
            }
            finally
            {
                if (bitmap != null)
                    bitmap.Dispose();
            }
        }

        public Texture(GraphicsSystem graphicsSystem, Bitmap bitmap)
        {
            _GraphicsSystem = graphicsSystem;

            _File = "";
            CreateTexture(bitmap);
        }

        public Texture(GraphicsSystem graphicsSystem, byte[] data, int width, int height)
        {
            _GraphicsSystem = graphicsSystem;

            _File = "";
            CreateTexture(data, width, height);
        }

        public string File
        {
            get { return _File; }
        }

        public int Width
        {
            get { return _Width; }
        }

        public int Height
        {
            get { return _Height; }
        }

        public void Dispose()
        {
            _GraphicsSystem.DisposeTexture(ref _ID);
        }

        internal uint ID
        {
            get { return _ID; }
        }

        private void CreateTexture(Bitmap bitmap)
        {
            BitmapData bd = null;
            int w = bitmap.Width;
            int h = bitmap.Height;
            byte[] data = null;

            try
            {
                w = bitmap.Width;
                h = bitmap.Height;
                data = new byte[w * h * 4];
                bd = bitmap.LockBits(new Rectangle(0, 0, w, h), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
                for (int y = 0, b = 0; y != h; y++)
                    for (int x = 0; x != w; x++)
                    {
                        int i = y * bd.Stride + x * 4;

                        data[b++] = Marshal.ReadByte(bd.Scan0, i + 2);
                        data[b++] = Marshal.ReadByte(bd.Scan0, i + 1);
                        data[b++] = Marshal.ReadByte(bd.Scan0, i);
                        data[b++] = Marshal.ReadByte(bd.Scan0, i + 3);
                    }
            }
            finally
            {
                if (bd != null)
                    bitmap.UnlockBits(bd);
            }
            CreateTexture(data, w, h);
        }

        private void CreateTexture(byte[] data, int width, int height)
        {
            _Width = width;
            _Height = height;

            _ID = 0;
            _GraphicsSystem.CreateTexture(data, width, height, ref _ID);
        }
    }
}
