﻿using System;

namespace Lib3D
{
    public class TransformSceneNode : SceneNode
    {
        public TransformSceneNode(GraphicsSystem graphicsSystem)
            : base(graphicsSystem)
        {
            Position = new Vector3(0, 0, 0);
            RotationMatrix = Matrix.CreateIdentity();
            Scale = new Vector3(1, 1, 1);
        }

        public TransformSceneNode(TransformSceneNode transformSceneNode)
            : base(transformSceneNode)
        {
            Position = transformSceneNode.Position;
            RotationMatrix = transformSceneNode.RotationMatrix;
            Scale = transformSceneNode.Scale;
        }

        public Vector3 Position
        {
            get;
            set;
        }

        public Matrix RotationMatrix
        {
            get;
            set;
        }

        public Vector3 Scale
        {
            get;
            set;
        }

        public void SetRotation(Vector3 rotation)
        {
            RotationMatrix =
                Matrix.CreateFromAxisAngle(new Vector3(1, 0, 0), rotation.X) *
                Matrix.CreateFromAxisAngle(new Vector3(0, 1, 0), rotation.Y) *
                Matrix.CreateFromAxisAngle(new Vector3(0, 0, 1), rotation.Z);
        }

        protected override void OnUpdate()
        {
            ModelMatrix = Matrix.CreateTranslation(Position) * RotationMatrix * Matrix.CreateScaling(Scale);
        }
    }
}
