﻿using System;
using System.Collections.Generic;

namespace Lib3D
{
    public class Triangulator
    {
        private List<Vector2> _Triangles;

        public Triangulator(List<Vector2> polygon)
        {
            Triangulate(polygon);
        }

        public List<Vector2> Triangles
        {
            get { return _Triangles; }
        }

        private void Triangulate(List<Vector2> polygon)
        {
            _Triangles = new List<Vector2>();

            if (polygon.Count < 3)
                return;

            List<Vector2> cPoly = new List<Vector2>(polygon);

            while (true)
            {
                int j = 0;
                int e = -1;
                int a = -1, b = -1, c = -1;

                while (j != cPoly.Count)
                {
                    bool ear = false;

                    if (IsConvex(cPoly, j))
                    {
                        a = j - 1;
                        b = j;
                        c = j + 1;

                        ear = true;

                        if (c >= cPoly.Count)
                            c = 0;
                        if (a < 0)
                            a = cPoly.Count - 1;
                        for (int k = 0; k != cPoly.Count; k++)
                        {
                            if (a != k && b != k && c != k && PointInTriangle(cPoly[k], cPoly[a], cPoly[b], cPoly[c]))
                            {
                                ear = false;
                                break;
                            }
                        }
                    }
                    if (ear)
                    {
                        e = j;
                        break;
                    }
                    j++;
                }
                if (e == -1)
                    break;

                _Triangles.Add(cPoly[a]);
                _Triangles.Add(cPoly[b]);
                _Triangles.Add(cPoly[c]);

                cPoly.RemoveAt(e);

                if (cPoly.Count < 3)
                    break;
            }
        }

        private double PolygonArea(Vector2[] polygon)
        {
            double area = 0;

            for (int i = 0; i != polygon.Length; i++)
            {
                Vector2 a = polygon[i];
                Vector2 b = polygon[(i + 1) % polygon.Length];

                area += a.X * b.Y;
                area -= a.Y * b.X;
            }
            return area / 2;
        }

        private bool IsConvex(List<Vector2> polygon, int i)
        {
            int a = i - 1;
            int b = i;
            int c = i + 1;

            if (c >= polygon.Count)
                c = 0;
            if (a < 0)
                a = polygon.Count - 1;

            double area = PolygonArea(new Vector2[] { polygon[a], polygon[b], polygon[c] });

            return area < 0;
        }

        private bool SameSide(Vector3 p1, Vector3 p2, Vector3 a, Vector3 b)
        {
            Vector3 cp1 = Vector3.Cross(b - a, p1 - a);
            Vector3 cp2 = Vector3.Cross(b - a, p2 - a);

            return Vector3.Dot(cp1, cp2) >= 0;
        }

        private bool PointInTriangle(Vector2 p, Vector2 a, Vector2 b, Vector2 c)
        {
            Vector3 p2 = new Vector3(p.X, p.Y, 0);
            Vector3 a2 = new Vector3(a.X, a.Y, 0);
            Vector3 b2 = new Vector3(b.X, b.Y, 0);
            Vector3 c2 = new Vector3(c.X, c.Y, 0);

            return SameSide(p2, a2, b2, c2) && SameSide(p2, b2, a2, c2) && SameSide(p2, c2, a2, b2);
        }
    }
}
