﻿using System;

namespace Lib3D
{
    public struct Vector2 : IEquatable<Vector2>
    {
        public double X;
        public double Y;

        public Vector2(double x, double y)
        {
            X = x;
            Y = y;
        }

        public Vector2(string value)
        {
            string[] tokens = value.Split(new char[] { ',', ' ', '\t', '\r', '\n' }, StringSplitOptions.RemoveEmptyEntries);

            X = double.Parse(tokens[0]);
            Y = double.Parse(tokens[1]);
        }

        public double Length()
        {
            return (double)Math.Sqrt(X * X + Y * Y);
        }

        public double LengthSquared()
        {
            return X * X + Y * Y;
        }

        public void Normalize()
        {
            double length = (double)Math.Sqrt(X * X + Y * Y);

            X /= length;
            Y /= length;
        }

        public override string ToString()
        {
            return string.Format("{0}, {1}", X, Y);
        }

        public bool Equals(Vector2 vec)
        {
            double x = Math.Abs(vec.X - X);
            double y = Math.Abs(vec.Y - Y);
            double t = 0.001f;

            return x < t && y < t;
        }

        public static Vector2 Normalize(Vector2 vec)
        {
            double length = (double)Math.Sqrt(vec.X * vec.X + vec.Y * vec.Y);

            return new Vector2(vec.X / length, vec.Y / length);
        }

        public static Vector2 operator +(Vector2 a, Vector2 b)
        {
            return new Vector2(a.X + b.X, a.Y + b.Y);
        }

        public static Vector2 operator -(Vector2 a, Vector2 b)
        {
            return new Vector2(a.X - b.X, a.Y - b.Y);
        }

        public static Vector2 operator -(Vector2 vec)
        {
            return new Vector2(-vec.X, -vec.Y);
        }

        public static Vector2 operator *(Vector2 a, Vector2 b)
        {
            return new Vector2(a.X * b.X, a.Y * b.Y);
        }

        public static Vector2 operator *(Vector2 a, double b)
        {
            return new Vector2(a.X * b, a.Y * b);
        }

        public static Vector2 operator *(double a, Vector2 b)
        {
            return new Vector2(a * b.X, a * b.Y);
        }

        public static Vector2 operator /(Vector2 a, Vector2 b)
        {
            return new Vector2(a.X / b.X, a.Y / b.Y);
        }

        public static Vector2 operator /(Vector2 a, double b)
        {
            return new Vector2(a.X / b, a.Y / b);
        }

        public static double Dot(Vector2 a, Vector2 b)
        {
            return a.X * b.X + a.Y * b.Y;
        }
    }
}
