﻿using System;

namespace Lib3D
{
    public struct Vector3 : IEquatable<Vector3>
    {
        public double X;
        public double Y;
        public double Z;

        public Vector3(double x, double y, double z)
        {
            X = x;
            Y = y;
            Z = z;
        }

        public Vector3(string value)
        {
            string[] tokens = value.Split(new char[] { ',', ' ', '\t', '\r', '\n' }, StringSplitOptions.RemoveEmptyEntries);

            X = double.Parse(tokens[0]);
            Y = double.Parse(tokens[1]);
            Z = double.Parse(tokens[2]);
        }

        public void CopyTo(float[] array, float pad)
        {
            array[0] = (float)X;
            array[1] = (float)Y;
            array[2] = (float)Z;
            array[3] = (float)pad;
        }

        public void CopyTo(double[] array, double pad)
        {
            array[0] = X;
            array[1] = Y;
            array[2] = Z;
            array[3] = pad;
        }

        public double Length()
        {
            return (double)Math.Sqrt(X * X + Y * Y + Z * Z);
        }

        public double LengthSquared()
        {
            return X * X + Y * Y + Z * Z;
        }

        public void Normalize()
        {
            double length = (double)Math.Sqrt(X * X + Y * Y + Z * Z);

            X /= length;
            Y /= length;
            Z /= length;
        }

        public override string ToString()
        {
            return string.Format("{0}, {1}, {2}", X, Y, Z);
        }

        public bool Equals(Vector3 vec)
        {
            double x = Math.Abs(vec.X - X);
            double y = Math.Abs(vec.Y - Y);
            double z = Math.Abs(vec.Z - Z);
            double t = 0.001f;

            return x < t && y < t && z < t;
        }

        public static Vector3 Normalize(Vector3 vec)
        {
            double length = (double)Math.Sqrt(vec.X * vec.X + vec.Y * vec.Y + vec.Z * vec.Z);

            return new Vector3(vec.X / length, vec.Y / length, vec.Z / length);
        }

        public static Vector3 Cross(Vector3 a, Vector3 b)
        {
            double x = a.Y * b.Z - a.Z * b.Y;
            double y = a.Z * b.X - a.X * b.Z;
            double z = a.X * b.Y - a.Y * b.X;

            return new Vector3(x, y, z);
        }

        public static Vector3 operator +(Vector3 a, Vector3 b)
        {
            return new Vector3(a.X + b.X, a.Y + b.Y, a.Z + b.Z);
        }

        public static Vector3 operator -(Vector3 a, Vector3 b)
        {
            return new Vector3(a.X - b.X, a.Y - b.Y, a.Z - b.Z);
        }

        public static Vector3 operator -(Vector3 vec)
        {
            return new Vector3(-vec.X, -vec.Y, -vec.Z);
        }

        public static Vector3 operator *(Vector3 a, Vector3 b)
        {
            return new Vector3(a.X * b.X, a.Y * b.Y, a.Z * b.Z);
        }

        public static Vector3 operator *(Vector3 a, double b)
        {
            return new Vector3(a.X * b, a.Y * b, a.Z * b);
        }

        public static Vector3 operator *(double a, Vector3 b)
        {
            return new Vector3(a * b.X, a * b.Y, a * b.Z);
        }

        public static Vector3 operator /(Vector3 a, Vector3 b)
        {
            return new Vector3(a.X / b.X, a.Y / b.Y, a.Z / b.Z);
        }

        public static Vector3 operator /(Vector3 a, double b)
        {
            return new Vector3(a.X / b, a.Y / b, a.Z / b);
        }

        public static double Dot(Vector3 a, Vector3 b)
        {
            return a.X * b.X + a.Y * b.Y + a.Z * b.Z;
        }
    }
}

