﻿using System;

namespace Lib3D
{
    public struct Vector4 : IEquatable<Vector4>
    {
        public double X;
        public double Y;
        public double Z;
        public double W;

        public Vector4(double x, double y, double z, double w)
        {
            X = x;
            Y = y;
            Z = z;
            W = w;
        }

        public Vector4(string value)
        {
            string[] tokens = value.Split(new char[] { ',', ' ', '\t', '\r', '\n' }, StringSplitOptions.RemoveEmptyEntries);

            X = double.Parse(tokens[0]);
            Y = double.Parse(tokens[1]);
            Z = double.Parse(tokens[2]);
            if (tokens.Length > 3)
                W = double.Parse(tokens[3]);
            else
                W = 1;
        }

        public void CopyTo(float[] array)
        {
            array[0] = (float)X;
            array[1] = (float)Y;
            array[2] = (float)Z;
            array[3] = (float)W;
        }

        public void CoptTo(double[] array)
        {
            array[0] = X;
            array[1] = Y;
            array[2] = Z;
            array[3] = W;
        }

        public double Length()
        {
            return (double)Math.Sqrt(X * X + Y * Y + Z * Z + W * W);
        }

        public double LengthSquared()
        {
            return X * X + Y * Y + Z * Z + W * W;
        }

        public void Normalize()
        {
            double length = (double)Math.Sqrt(X * X + Y * Y + Z * Z + W * W);

            X /= length;
            Y /= length;
            Z /= length;
            W /= length;
        }

        public override string ToString()
        {
            return string.Format("{0}, {1}, {2}, {3}", X, Y, Z, W);
        }

        public bool Equals(Vector4 vec)
        {
            double x = Math.Abs(vec.X - X);
            double y = Math.Abs(vec.Y - Y);
            double z = Math.Abs(vec.Z - Z);
            double w = Math.Abs(vec.W - W);
            double t = 0.001f;

            return x < t && y < t && z < t && w < t;
        }

        public static Vector4 Normalize(Vector4 vec)
        {
            double length = (double)Math.Sqrt(vec.X * vec.X + vec.Y * vec.Y + vec.Z * vec.Z + vec.W * vec.W);

            return new Vector4(vec.X / length, vec.Y / length, vec.Z / length, vec.W / length);
        }

        public static Vector4 operator +(Vector4 a, Vector4 b)
        {
            return new Vector4(a.X + b.X, a.Y + b.Y, a.Z + b.Z, a.W + b.W);
        }

        public static Vector4 operator -(Vector4 a, Vector4 b)
        {
            return new Vector4(a.X - b.X, a.Y - b.Y, a.Z - b.Z, a.W - b.W);
        }

        public static Vector4 operator -(Vector4 vec)
        {
            return new Vector4(-vec.X, -vec.Y, -vec.Z, -vec.W);
        }

        public static Vector4 operator *(Vector4 a, Vector4 b)
        {
            return new Vector4(a.X * b.X, a.Y * b.Y, a.Z * b.Z, a.W * b.W);
        }

        public static Vector4 operator *(Vector4 a, double b)
        {
            return new Vector4(a.X * b, a.Y * b, a.Z * b, a.W * b);
        }

        public static Vector4 operator *(double a, Vector4 b)
        {
            return new Vector4(a * b.X, a * b.Y, a * b.Z, a * b.W);
        }

        public static Vector4 operator /(Vector4 a, Vector4 b)
        {
            return new Vector4(a.X / b.X, a.Y / b.Y, a.Z / b.Z, a.W / b.W);
        }

        public static Vector4 operator /(Vector4 a, double b)
        {
            return new Vector4(a.X / b, a.Y / b, a.Z / b, a.W / b);
        }

        public static double Dot(Vector4 a, Vector4 b)
        {
            return a.X * b.X + a.Y * b.Y + a.Z * b.Z + a.W * b.W;
        }
    }
}


